@@ -6,8 +6,8 @@ sub Language::FormulaEngine::Compiler::_CleanEval::_clean_eval {
 	use strict; # these apply to the contents of the eval, too.
 	use warnings;
 	# Arguments are ($compiler, $perl_code)
-	my $default_namespace= shift->namespace;
-	eval shift;
+	my $default_namespace= $_[0]->namespace; # referred to by evals
+	eval $_[1];
 }
 
 use Moo;
@@ -18,7 +18,7 @@ use Sub::Util 'subname', 'set_subname';
 use namespace::clean;
 
 # ABSTRACT: Compile a parse tree into perl code
-our $VERSION = '0.06'; # VERSION
+our $VERSION = '0.07'; # VERSION
 
 *_clean_eval= *Language::FormulaEngine::Compiler::_CleanEval::_clean_eval;
 
@@ -53,7 +53,7 @@ sub _trigger_output_api {
 sub _build__optimize_var_access {
 	my $self= shift;
 	return $self->optimize_var_access if defined $self->optimize_var_access;
-	return $self->output_api eq 'function_of_vars'
+	return ($self->output_api =~ /^function_of_vars/)
 		&& $self->namespace->can('get_value') == Language::FormulaEngine::Namespace->can('get_value');
 }
 
@@ -62,7 +62,7 @@ sub BUILD {
 	# Handle back-compat for initial broken version of this feature.
 	# There is no longer any reason to set variables_via_namespace to true, because true is the default.
 	# So if a user does that, they might be asking for the 'output_api => "function_of_namespace"'
-	if ($self->variables_via_namespace) {
+	if (defined $self->variables_via_namespace) {
 		carp "variables_via_namespace is deprecated.  See 'output_api' and 'optimize_var_access'";
 		$self->output_api('function_of_namespace');
 		$self->optimize_var_access(0);
@@ -76,7 +76,7 @@ sub compile {
 	$self->reset;
 	try {
 		$self->code_body($self->perlgen($parse_tree));
-		$ret= $self->generate_coderef_wrapper($self->code_body);
+		$ret= $self->generate_coderef_wrapper($self->code_body, $subname);
 	}
 	catch {
 		chomp unless ref $_;
@@ -97,40 +97,9 @@ sub reset {
 sub generate_coderef_wrapper {
 	my ($self, $perl, $subname)= @_;
 	$self->error(undef);
-	my @code= (
-		'# line '.(__LINE__+1),
-		'sub {',
-		'  use warnings FATAL => qw( uninitialized numeric );',
-	);
-	if ($self->output_api eq 'function_of_vars') {
-		if ($self->_optimize_var_access) {
-			push @code,
-				'# line '.(__LINE__+1),
-				'  my $namespace= $default_namespace;',
-				'  my $vars= $namespace->variables;',
-				'  $vars= { %$vars, (@_ == 1 && ref $_[0] eq "HASH"? %{$_[0]} : @_) } if @_;'
-		}
-		else {
-			push @code,
-				'# line '.(__LINE__+1),
-				'  my $namespace= @_ == 0? $default_namespace',
-				'    : $default_namespace->clone_and_merge(variables => (@_ == 1 && ref $_[0] eq "HASH"? $_[0] : { @_ }));';
-		}
-	} elsif ($self->output_api eq 'function_of_namespace') {
-		push @code,
-			'# line '.(__LINE__+1),
-			'  my $namespace= @_ == 0? $default_namespace',
-			'    : @_ == 1 && Scalar::Util::blessed($_[0])? $_[0]',
-			'    : $default_namespace->clone_and_merge(@_);';
-		push @code,
-			'  my $vars= $namespace->variables;'
-			if $self->_optimize_var_access;
-	}
-	else {
-		croak "Unhandled output_api = '".$self->output_api."'";
-	}
-
-	my $code= join "\n", @code, '# line 0 "compiled formula"', $perl, '}';
+	my $wrapper_method= $self->can('_output_wrapper__'.$self->output_api)
+		or Carp::croak("Unsupported output_api='".$self->output_api."'");
+	my $code= join "\n", $self->$wrapper_method(qq{# line 0 "compiled formula"\n$perl});
 	my $ret;
 	{
 		local $@= undef;
@@ -143,6 +112,62 @@ sub generate_coderef_wrapper {
 	return $ret;
 }
 
+sub _output_wrapper__function_of_vars {
+	my ($self, $code)= @_;
+	return $self->_optimize_var_access? (
+		'# line '.(__LINE__+1),
+		'my $namespace= $default_namespace;',
+		'sub {',
+		'  use warnings FATAL => qw( uninitialized numeric );',
+		'  my $vars= $namespace->variables;',
+		'  $vars= { %$vars, (@_ == 1? %{$_[0]} : @_) } if @_;',
+		$code,
+		'}'
+	) : (
+		'# line '.(__LINE__+1),
+		'sub {',
+		'  use warnings FATAL => qw( uninitialized numeric );',
+		'  my $namespace= @_ == 0? $default_namespace',
+		'    : $default_namespace->clone_and_merge(variables => (@_ == 1 && ref $_[0] eq "HASH"? $_[0] : { @_ }));',
+		$code,
+		'}'
+	)
+}
+
+sub _output_wrapper__function_of_vars_no_default {
+	my ($self, $code)= @_;
+	return $self->_optimize_var_access? (
+		'# line '.(__LINE__+1),
+		'my $namespace= $default_namespace;',
+		'sub {',
+		'  use warnings FATAL => qw( uninitialized numeric );',
+		'  my $vars= @_ == 1? $_[0] : { @_ };',
+		$code,
+		'}'
+	) : (
+		'# line '.(__LINE__+1),
+		'sub {',
+		'  use warnings FATAL => qw( uninitialized numeric );',
+		'  my $namespace= ref($default_namespace)->new(variables => (@_ == 1? $_[0] : { @_ }));',
+		$code,
+		'}'
+	)
+}
+
+sub _output_wrapper__function_of_namespace {
+	my ($self, $code)= @_;
+	return
+		'# line '.(__LINE__+1),
+		'sub {',
+		'  use warnings FATAL => qw( uninitialized numeric );',
+		'  my $namespace= @_ == 0? $default_namespace',
+		'    : @_ == 1 && Scalar::Util::blessed($_[0])? $_[0]',
+		'    : $default_namespace->clone_and_merge(@_);',
+		($self->_optimize_var_access? '  my $vars= $namespace->variables;' : ()),
+		$code,
+		'}'
+}
+
 
 sub perlgen {
 	my ($self, $node)= @_;
@@ -199,7 +224,7 @@ sub perlgen_var_access {
 
 sub perlgen_string_literal {
 	my ($self, $string)= @_;
-	$string =~ s/([\0-\x1F\x7f"\@\$\\])/ sprintf("\\x%02x", ord $1) /gex;
+	$string =~ s/([\0-\x1F\x7f"\@\$\%\\])/ sprintf("\\x%02x", ord $1) /gex;
 	return qq{"$string"};
 }
 
@@ -224,33 +249,60 @@ Language::FormulaEngine::Compiler - Compile a parse tree into perl code
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 DESCRIPTION
 
+The Compiler object takes a parse tree and generates perl code from it, and also evals
+that perl code into a compiled coderef.  However, most of the code generation is handled
+by the L<Language::FormulaEngine::Namespace> object.  The namespace object must be
+available during compilation.  In the default scenario, the namespace object will be
+consulted for default constants and "global" variables, and then the generated coderef
+will accept additional variables to overlay on those global variables, but the Namespace
+will still get to determine how to access that pool of combined variables at execution.
+
+For alternative strategies, see the C<output_api> attribute below.
+
 =head1 ATTRIBUTES
 
 =head2 namespace
 
-Namespace to use for looking up functions, converting functions to perl code, and symbolic
-constants.  The namespace will also be bound into the coderefs which get compiled, so any
-change to the variables (not constants) of the namespace will be visible to compiled formulas.
+Instance of L<Language::FormulaEngine::Namespace> to use for looking up functions,
+converting functions to perl code, and symbolic constants.  This namespace object will
+be referenced by the coderefs, and can act as a "global scope".
 
 =head2 output_api
 
 Determines the function signature of the coderef that will be generated.  Currently supported
-values are
+values are:
 
 =over 25
 
 =item C<"function_of_vars">
 
-Compiles as C<< $return = my_formula(%vars) >> or C<< $return = my_formula(\%vars) >>
+  # generated coderef signature:
+  $value = $coderef->(%vars);
+  $value = $coderef->(\%vars);
+
+The list or hashref of variables will get overlaid ontop of the set of variables of the
+namespace (by creating a derived namespace), and the namespace C<get_value> method will
+retrieve values from them.
+
+=item C<"function_of_vars_no_default">
+
+Same signature as "function_of_vars" but current C<< $namespace->variables >> are ignored
+and either a new empty namespace is created each time, or (if C<optimize_var_access> is
+active) no namespace will be used at all and vars will be read directly from a hashref.
 
 =item C<"function_of_namespace">
 
-Compiles as C<< $return = my_formula($namespace) >> or C<< $return = my_formula(%namespace_attrs) >>
-where C<%namespace_attrs> get passed to C<clone_and_merge> of L</namespace>.
+  # generated coderef signature:
+  $value= $coderef->();
+  $value= $coderef->($namespace);
+  $value= $coderef->(\%namespace_attributes);
+
+This either uses the supplied namespace *instead* of the default, or merges attributes with
+the default namespace via L<Language::FormulaEngine::Namespace/clone_and_merge>.
 
 =back
 
@@ -262,7 +314,7 @@ bypassing C<get_value>.
 
 If this attribute is not set, the compilation will default to using the optimization if the
 L</namespace> is using the default implementation of C<get_value> (i.e. has not been overridden
-by a subclass) and if output_api is C<"function_of_vars">.
+by a subclass) and the coderefs are a function of variables.
 
 =head2 error
 
@@ -280,10 +332,9 @@ wrapped with the coderef boilerplate.
 Compile a parse tree, returning a coderef.  Any references to functions will be immeditely
 looked up within the L</namespace>.  Any references to constants in the L</namespace> will be
 inlined into the generated perl.  Any other symbol is assumed to be a variable, and will be
-looked up from the L</namespace> at the time the formula is invoked.  The generated coderef
-takes parameters of overrides for the set of variables in the namespace:
+looked up from the L</namespace> at the time the formula is invoked.
 
-  $value= $compiled_sub->(%vars); # vars are optional
+See attribute C<output_api> for the signature and behavior of this coderef.
 
 Because the generated coderef contains a reference to the namespace, be sure never to store
 one of the coderefs into that namespace object, else you get a memory leak.
@@ -304,22 +355,28 @@ according to L</output_api>, and then evals the perl to create the coderef.
 
 On a compile failure, this returns C<undef> and puts the error message into L</error>.
 
-=head2 perlgen( $parse_node )
+=head2 perlgen
 
-Generate perl source code for a parse node.
+  $perl_code= $compiler->perlgen( $parse_node );
+
+Generate perl source code for a parse node.  This is a fragment, not a whole sub.
+The code should always be an expression which can be combined with other
+function calls.
 
 =head2 perlgen_var_access
 
   $compiler->perlgen_var_access($varname);
 
-Generate perl code to access a variable.  If L</variables_via_namespace> is true, this becomes
-a call to C<< $namespace->get_value($varname) >>.  Else it becomes a reference to the variables
-hashref C<< $vars->{$varname} >>.
+Generate perl code to access a variable.  Unless L</optimize_var_access> is true,
+this becomes a call to C<< $namespace->get_value($varname) >>, and the namespace
+decides how to interpret the variable at execution time.  If C<optimize_var_access>
+is enabled, this returns a reference to the C<$vars> hashref like C<< $vars->{$varname} >>.
 
 =head2 perlgen_string_literal
 
 Generate a perl string literal.  This wraps the string with double-quotes and escapes control
-characters and C<["\\\@\$]> using hex-escape notation.
+characters and C<["\\\@\$]> using hex-escape notation.  Hex escapes are chosen over simple
+backslash prefixes for extra security, in case of mistakes elsewhere in the generated code.
 
 =head2 perlgen_literal
 
@@ -332,7 +389,7 @@ Michael Conrad <mconrad@intellitree.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Michael Conrad, IntelliTree Solutions llc.
+This software is copyright (c) 2023 by Michael Conrad, IntelliTree Solutions llc.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
